import cli from '@battis/qui-cli';
import path from 'node:path';
import * as common from '../../common.js';
import * as Downloader from './Downloader.js';
export class Spider {
    downloader;
    constructor(options) {
        this.downloader = new Downloader.Downloader(options);
    }
    async download(snapshot, { outputOptions, ...options }) {
        const { outputPath, pretty } = outputOptions;
        if (!outputPath) {
            throw new common.Output.OutputError('Spider requires outputPath');
        }
        if (snapshot) {
            common.Debug.withGroupId(snapshot.SectionInfo?.Id || cli.colors.error('unknown'), 'Downloading supporting files');
            await this.traverse(snapshot, {
                host: snapshot.Metadata.Host,
                outputOptions,
                ...options,
                pathToComponent: path.basename(outputPath)
            });
            const indexName = `${snapshot.SectionInfo?.Id || 'index'}.json`;
            await common.Output.writeJSON(await common.Output.avoidOverwrite(path.join(outputPath, indexName)), snapshot, {
                pretty
            });
            common.Debug.withGroupId(snapshot.SectionInfo?.Id || cli.colors.error('unknown'), `Supporting files exported to ${cli.colors.url(outputPath)}/${cli.colors.value(indexName)}`);
            return indexName;
        }
        else {
            cli.log.warning('Could not downlod course content (no index available)');
            return undefined;
        }
    }
    async traverse(snapshotComponent, { pathToComponent, ...options }) {
        const { include, exclude, ignoreErrors } = options;
        if (Array.isArray(snapshotComponent)) {
            await Promise.allSettled(snapshotComponent.map(async (elt, i) => {
                await this.traverse(elt, {
                    pathToComponent: `${pathToComponent}[${i}]`,
                    ...options
                });
            }));
        }
        else {
            await Promise.allSettled(Object.keys(snapshotComponent).map(async (key) => {
                // TODO ideally catch null case before recursive invocation
                if (snapshotComponent[key] === null) {
                    return;
                }
                else if (typeof snapshotComponent[key] === 'object') {
                    await this.traverse(snapshotComponent[key], {
                        pathToComponent: `${pathToComponent}.${key}`,
                        ...options
                    });
                }
                else if (key === 'PhotoFilename' ||
                    key === 'ThumbFilename' ||
                    /Url$/i.test(key) ||
                    (/FilePath$/i.test(key) &&
                        !snapshotComponent[key].endsWith('/'))) {
                    if (snapshotComponent[key] &&
                        (!include ||
                            include.reduce((included, regex) => included || regex.test(snapshotComponent[key]), false)) &&
                        (!exclude ||
                            exclude.reduce((excluded, regex) => excluded && !regex.test(snapshotComponent[key]), true))) {
                        let url = snapshotComponent[key];
                        if (key === 'ThumbFilename' &&
                            /^thumb_topic_[a-z0-9_.]/i.test(url)) {
                            url = `/ftpimages/:SchoolId/topics/${url}`;
                        }
                        else if (key === 'PhotoFilename' &&
                            /^thumb_user_[a-z0-9_.]/i.test(url)) {
                            url = `/ftpimages/:SchoolId/user/${url}`;
                        }
                        cli.log.debug({ pathToComponent, key, url });
                        try {
                            const item = await this.downloader.download(url, 'FriendlyFileName' in snapshotComponent &&
                                typeof snapshotComponent['FriendlyFileName'] === 'string'
                                ? snapshotComponent['FriendlyFileName']
                                : undefined);
                            snapshotComponent[key] = item;
                            cli.log.debug(`${pathToComponent}[${key}]: ${item.localPath || item.error}`);
                        }
                        catch (error) {
                            if (ignoreErrors) {
                                const message = `Download ${cli.colors.value(key)} ${cli.colors.url(snapshotComponent[key])} failed: ${error}`;
                                cli.log.error(message);
                                snapshotComponent[key] = {
                                    original: snapshotComponent[key],
                                    accessed: new Date(),
                                    error: message
                                };
                            }
                            else {
                                throw error;
                            }
                        }
                    }
                }
            }));
        }
    }
    async quit() {
        await this.downloader.quit();
    }
}
//# sourceMappingURL=Spider.js.map